/*
 * Copyright 2024 Luca Fulchir <luca.fulchir@runesauth.com>
 *
 * Licensed under the Apache License, Version 2.0 with LLVM exception (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License and of the exception at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     https://spdx.org/licenses/LLVM-exception.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

//!
//! Example of two package repositories, where one
//! extends ancd changes another
//!

use crate::libt;
//use ::std::{boxed::Box, vec::Vec};

crate::libt::moduse! {
    one
    two
    three
}

///
/// This is a base Package List
///
#[derive(::std::default::Default)]
pub struct Pkgs1 {
    base: libt::PkgsEmpty,
}
/// eventually PkgsList and deref will be replaced by macros
impl libt::PkgsList for Pkgs1 {}
impl ::std::ops::Deref for Pkgs1 {
    type Target = libt::PkgsEmpty;
    fn deref(&self) -> &libt::PkgsEmpty {
        &self.base
    }
}

impl Pkgs1 {
    /// Packages like this can become a  macro
    pub fn one(&self) -> One {
        One::new()
    }
    /// Packages like this can become a  macro
    pub fn two(&self) -> Two {
        Two::new()
    }
}

///
/// This is a Package List that extends and changes Pkgs1
///
#[derive(::std::default::Default)]
pub struct Pkgs2 {
    base: Pkgs1,
}
/// eventually PkgsList and deref will be replaced by macros
impl libt::PkgsList for Pkgs2 {}

impl ::std::ops::Deref for Pkgs2 {
    type Target = Pkgs1;
    fn deref(&self) -> &Pkgs1 {
        &self.base
    }
}

impl Pkgs2 {
    /// example of option override in a package
    pub fn two(&self) -> Two {
        let mut t = Two::new();
        t.my_attr = 42;
        t
    }
    /// Packages like this can become a  macro
    pub fn three(&self) -> Three {
        Three::new()
    }
}
